@@ -1,5 +1,5 @@
 # 2ping - A bi-directional ping utility
-# Copyright (C) 2010-2017 Ryan Finnie
+# Copyright (C) 2010-2018 Ryan Finnie
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -202,6 +202,10 @@ def parse_args(argv=None):
         '--srv', action='store_true',
         help=_('lookup SRV records in client mode'),
     )
+    parser.add_argument(
+        '--srv-service', type=str, default='2ping',
+        help=_('service name for SRV lookups'),
+    )
 
     # ping-compatible ignored options
     for opt in 'b|B|d|L|n|R|r|U'.split('|'):
